import { Method, Header } from '@shopify/network';
import { WebhookHeader } from './types.mjs';

let DeliveryMethod;

(function (DeliveryMethod) {
  DeliveryMethod["Http"] = "http";
  DeliveryMethod["EventBridge"] = "eventbridge";
})(DeliveryMethod || (DeliveryMethod = {}));

async function registerWebhook({
  address,
  topic,
  accessToken,
  shop,
  apiVersion,
  deliveryMethod = DeliveryMethod.Http
}) {
  const response = await fetch(`https://${shop}/admin/api/${apiVersion}/graphql.json`, {
    method: Method.Post,
    body: buildQuery(topic, address, deliveryMethod),
    headers: {
      [WebhookHeader.AccessToken]: accessToken,
      [Header.ContentType]: 'application/graphql'
    }
  });
  const result = await response.json();
  return {
    success: isSuccess(result, deliveryMethod),
    result
  };
}

function isSuccess(result, deliveryMethod) {
  switch (deliveryMethod) {
    case DeliveryMethod.Http:
      return Boolean(result.data && result.data.webhookSubscriptionCreate && result.data.webhookSubscriptionCreate.webhookSubscription);

    case DeliveryMethod.EventBridge:
      return Boolean(result.data && result.data.eventBridgeWebhookSubscriptionCreate && result.data.eventBridgeWebhookSubscriptionCreate.webhookSubscription);
  }
}

function buildQuery(topic, address, deliveryMethod) {
  let mutationName;
  let webhookSubscriptionArgs;

  switch (deliveryMethod) {
    case DeliveryMethod.Http:
      mutationName = 'webhookSubscriptionCreate';
      webhookSubscriptionArgs = `{callbackUrl: "${address}"}`;
      break;

    case DeliveryMethod.EventBridge:
      mutationName = 'eventBridgeWebhookSubscriptionCreate';
      webhookSubscriptionArgs = `{arn: "${address}"}`;
      break;
  }

  return `
    mutation webhookSubscriptionCreate {
      ${mutationName}(topic: ${topic}, webhookSubscription: ${webhookSubscriptionArgs}) {
        userErrors {
          field
          message
        }
        webhookSubscription {
          id
        }
      }
    }
  `;
}

export { DeliveryMethod, registerWebhook };
